==> Demo_STC12C5A60S2_ST7789V2_3-Line_Hardware_SPI.zip: Demo_STC12C5A60S2_ST7789V2_3-Line_Hardware_SPI/user/main.c <==
/**********************************************************************
 * ֻѧϰʹãδɣκ;
 * ӲƬSTC12C5A60S2,Ƶ12MHZ
 * ˵ST7789V 3ӲSPI
 
 ----------------------------------------------------
 |ǧһʾƼ޹˾                        |
 |˾վ:www.qytft.com                            |
 |Ավhttps://shop580133616.taobao.com        |
 |̻() :+86 0755-27908130                     |
 |ҵѯ:18823372746 (쾭 WXͬ)              |
 |ҵѯ:15989313508 (뾭 WXͬ)              |
 ----------------------------------------------------

 ----------------------------------------------------
 |˾ṩ֧֣κμ⻶ӭʱѧϰ    |
 |֧QQ:3597321058                             |
 |֧:qyxskj@126.com                       |
 |֧:13723763193ܹ WXͬţ               |
 |QQȺ:792027219                            |
 ----------------------------------------------------

 * ˵:
 ========================ԴŽ==================================        
 |   LCDģ           Ƭ                                     |
 |     VCC      --->      5V        //Դ                            |
 |     GND      --->      GND       //Դ                          | 
 |====================ƼŽ==============================|
 |   LCDģ 			     Ƭ                                     |
 |     CS       --->      P13       //ҺƬѡźţ͵ƽЧ|
 |     RST      --->      P33       //Һλźţ͵ƽЧ|
 |     SCL      --->      P17       //ҺSPIʱź           |
 |     SDI      --->      P15       //ҺSPIдź         |
 |     SDO      --->      P16       //ҺSPI߶ź         |
 ======================================================================
 
 * :2020/5/13
 * 汾V1.0
 * ȨУؾ
 * Copyright(C) ǧһʾƼ޹˾ 2020-2035
 * All rights reserved
**********************************************************************/
#include "main.h"

u16 width=LCD_W;
u16 height=LCD_H;
u16 xoffset=0;
u16 yoffset=0;

void delay_ms(int count)  // /* X1ms */
{
    int i,j;
	for(i=0;i<count;i++)
        for(j=0;j<1000;j++);
}

void delay_us(int count)  // /* X1us */
{
    int i,j;
        for(i=0;i<count;i++)
            for(j=0;j<1;j++);
}

void SPI_init(void)	
{
	SPCTL = (SSIG << 7)+(SPEN << 6)+(DORD << 5)+(MSTR << 4)+(CPOL << 3)+(CPHA << 2)+SPEED_16;
}

void SPI_WriteByte(u16 byte)
{			
	SPDAT = (byte>>8)&0xFF;		//һֽ
	while((SPSTAT & SPIF)==0)	;	//ȴ
	SPSTAT = SPIF+WCOL;			//0 SPIFWCOL־
	SPDAT = byte&0xFF;		//һֽ
	while((SPSTAT & SPIF)==0)	;	//ȴ
	SPSTAT = SPIF+WCOL;			//0 SPIFWCOL־
}

void LCD_WR_REG(u8 Reg)	 
{	
	LCD_CS=0;
	SPI_WriteByte((Reg<<7)|0x0000);
	LCD_CS=1;
} 

void LCD_WR_DATA(u8 Data)
{
	LCD_CS=0;
	SPI_WriteByte((Data<<7)|0x8000);
	LCD_CS=1;
}

void LCD_WriteReg(u8 LCD_Reg, u8 LCD_RegValue)
{
	LCD_WR_REG(LCD_Reg);
	LCD_WR_DATA(LCD_RegValue);
}

void LCD_SetWindows(u16 xStar, u16 yStar,u16 xEnd,u16 yEnd)
{	
	LCD_WR_REG(SET_X_ADD_CMD);	
	LCD_WR_DATA((xStar+xoffset)>>8);
	LCD_WR_DATA(xStar+xoffset);		
	LCD_WR_DATA((xEnd+xoffset)>>8);
	LCD_WR_DATA(xEnd+xoffset);
	LCD_WR_REG(SET_Y_ADD_CMD);	
	LCD_WR_DATA((yStar+yoffset)>>8);
	LCD_WR_DATA(yStar+yoffset);		
	LCD_WR_DATA((yEnd+yoffset)>>8);
	LCD_WR_DATA(yEnd+yoffset);

	LCD_WR_REG(WR_RAM_CMD);	//ʼдGRAM				
}   

void LCD_Clear(u16 Color)
{
	u16 i,j;
	LCD_SetWindows(0,0,width-1,height-1);	
  for(i=0;i<width;i++)
	{
	  for (j=0;j<height;j++)
	  {
			LCD_WR_DATA(Color>>8);
			LCD_WR_DATA(Color);
	  }
	}
}

void LCD_Reset(void)
{	
	LCD_RESET=0;
	delay_ms(20);
	LCD_RESET=1;
	delay_ms(20);
}

void LCD_direction(u8 direction)
{ 
	switch(direction%4){		  
		case 0:						 	 		
			width=LCD_W;
			height=LCD_H;
			xoffset=0;
			yoffset=0;			
			LCD_WriteReg(0x36,0);//BGR==1,MY==0,MX==0,MV==0
		break;
		case 1:
			width=LCD_H;
			height=LCD_W;
			xoffset=0;
			yoffset=0;
			LCD_WriteReg(0x36,(1<<6)|(1<<5));//BGR==1,MY==1,MX==0,MV==1
		break;
		case 2:						 	 		
			width=LCD_W;
			height=LCD_H;
      xoffset=0;
			yoffset=80;				
			LCD_WriteReg(0x36,(1<<6)|(1<<7));//BGR==1,MY==0,MX==0,MV==0
		break;
		case 3:
			width=LCD_H;
			height=LCD_W;
			xoffset=80;
			yoffset=0;
			LCD_WriteReg(0x36,(1<<7)|(1<<5));//BGR==1,MY==1,MX==0,MV==1
		break;	
		default:break;
	}		
}

void LCD_Init(void)
{
	LCD_Reset(); //ʼ֮ǰλ
//*************ST7789Vʼ**********//
	delay_ms(120);
//--------------------------------Display and color format setting----------------------------//
	LCD_WR_REG(0x36);
	LCD_WR_DATA(0x00);
	LCD_WR_REG(0x3a);
	LCD_WR_DATA(0x05);
	LCD_WR_REG(0x21);
//--------------------------------ST7789S Frame rate setting----------------------------------//
	LCD_WR_REG(0xb2);
	LCD_WR_DATA(0x0c);
	LCD_WR_DATA(0x0c);
	LCD_WR_DATA(0x00);
	LCD_WR_DATA(0x33);
	LCD_WR_DATA(0x33);
	LCD_WR_REG(0xb7);
	LCD_WR_DATA(0x35);
//---------------------------------ST7789S Power setting--------------------------------------//
	LCD_WR_REG(0xbb);
	LCD_WR_DATA(0x1f);
	LCD_WR_REG(0xc0);
	LCD_WR_DATA(0x2c);
	LCD_WR_REG(0xc2);
	LCD_WR_DATA(0x01);
	LCD_WR_REG(0xc3);
	LCD_WR_DATA(0x12);
	LCD_WR_REG(0xc4);
	LCD_WR_DATA(0x20);
	LCD_WR_REG(0xc6);
	LCD_WR_DATA(0x0f);
	LCD_WR_REG(0xd0);
	LCD_WR_DATA(0xa4);
	LCD_WR_DATA(0xa1);
//--------------------------------ST7789S gamma setting---------------------------------------//
	LCD_WR_REG(0xe0);
	LCD_WR_DATA(0xd0);
	LCD_WR_DATA(0x0e);
	LCD_WR_DATA(0x14);
	LCD_WR_DATA(0x0b);
	LCD_WR_DATA(0x0a);
	LCD_WR_DATA(0x16);
	LCD_WR_DATA(0x3b);
	LCD_WR_DATA(0x55);
	LCD_WR_DATA(0x52);
	LCD_WR_DATA(0x39);
	LCD_WR_DATA(0x14);
	LCD_WR_DATA(0x15);
	LCD_WR_DATA(0x31);
	LCD_WR_DATA(0x35);
	LCD_WR_REG(0xe1);
	LCD_WR_DATA(0xd0);
	LCD_WR_DATA(0x0e);
	LCD_WR_DATA(0x14);
	LCD_WR_DATA(0x0a);
	LCD_WR_DATA(0x09);
	LCD_WR_DATA(0x06);
	LCD_WR_DATA(0x3c);
	LCD_WR_DATA(0x33);
	LCD_WR_DATA(0x53);
	LCD_WR_DATA(0x08);
	LCD_WR_DATA(0x15);
	LCD_WR_DATA(0x15);
	LCD_WR_DATA(0x32);
	LCD_WR_DATA(0x35);
/*Sleep out*/
	LCD_WR_REG( 0x11);
	delay_ms(60);
/*Display on*/
	LCD_WR_REG( 0x29);		
//	LCD_BL=1;//	 
}

//
void main(void)
{ 
	u8 i=0;
	//STC_12LE5A60S2ƬP3.2ΪڿƱ
	//ͨC51Ƭ޴
	//P3M1 &= ~(1<<2),P3M0 |=  (1<<2); 
	SPI_init(); //ʼӲSPI
	//Һʼ
	LCD_Init();

	//ѭи	
	while(1)
	{
		LCD_direction(i++);
		LCD_Clear(WHITE);delay_ms(1000);
		LCD_Clear(BLACK);delay_ms(1000);
		LCD_Clear(RED);delay_ms(1000);
		LCD_Clear(GREEN);delay_ms(1000);
		LCD_Clear(BLUE);delay_ms(1000);
		if(i>3)
		{
			i=0;
		}
	}   
}

==> Demo_STC12C5A60S2_ST7789V2_4-Line_Hardware_SPI.zip: Demo_STC12C5A60S2_ST7789V2_4-Line_Hardware_SPI/user/main.c <==
/**********************************************************************
 * ֻѧϰʹãδɣκ;
 * ӲƬSTC12C5A60S2,Ƶ12MHZ
 * ˵ST7789V 4ӲSPI
 
 ----------------------------------------------------
 |ǧһʾƼ޹˾                        |
 |˾վ:www.qytft.com                            |
 |Ավhttps://shop580133616.taobao.com        |
 |̻() :+86 0755-27908130                     |
 |ҵѯ:18823372746 (쾭 WXͬ)              |
 |ҵѯ:15989313508 (뾭 WXͬ)              |
 ----------------------------------------------------

 ----------------------------------------------------
 |˾ṩ֧֣κμ⻶ӭʱѧϰ    |
 |֧QQ:3597321058                             |
 |֧:qyxskj@126.com                       |
 |֧:13723763193ܹ WXͬţ               |
 |QQȺ:792027219                            |
 ----------------------------------------------------

 * ˵:
 ========================ԴŽ=======================================================        
 |   LCDģ           Ƭ                                                          |
 |     VCC      --->      5V        //Դ                                                 |
 |     GND      --->      GND       //Դ                                               |  
 |====================ƼŽ===================================================|
 |   LCDģ  			   Ƭ                                                          |
 |     CS       --->      P13       //ҺƬѡźţ͵ƽЧ                     |
 |     RS       --->      P12       //Һ/ݿźţ͵ƽߵƽݣ|
 |     RST      --->      P33       //Һλźţ͵ƽЧ                     |
 |     SCL      --->      P17       //ҺSPIʱź                                |
 |     SDI      --->      P15       //ҺSPIдź                              |
 |     SDO      --->      P16       //ҺSPI߶ź                              |
 ===========================================================================================
 
 * :2020/5/13
 * 汾V1.0
 * ȨУؾ
 * Copyright(C) ǧһʾƼ޹˾ 2020-2035
 * All rights reserved
**********************************************************************/
#include "main.h"

u16 width=LCD_W;
u16 height=LCD_H;

void delay_ms(int count)  // /* X1ms */
{
    int i,j;
	for(i=0;i<count;i++)
        for(j=0;j<1000;j++);
}

void delay_us(int count)  // /* X1us */
{
    int i,j;
        for(i=0;i<count;i++)
            for(j=0;j<1;j++);
}

void SPI_init(void)	
{
	SPCTL = (SSIG << 7)+(SPEN << 6)+(DORD << 5)+(MSTR << 4)+(CPOL << 3)+(CPHA << 2)+SPEED_16;
}

void SPI_WriteByte(u8 byte)
{			
	SPDAT = byte;		//һֽ
	while((SPSTAT & SPIF)==0)	;	//ȴ
	SPSTAT = SPIF+WCOL;			//0 SPIFWCOL־
}

void LCD_WR_REG(u8 Reg)	 
{	
	LCD_CS=0;
	LCD_RS=0;
	SPI_WriteByte(Reg);
	LCD_CS=1;
} 

void LCD_WR_DATA(u8 Data)
{
	LCD_CS=0;
	LCD_RS=1;
	SPI_WriteByte(Data);
	LCD_CS=1;
}

void LCD_WriteReg(u8 LCD_Reg, u8 LCD_RegValue)
{
	LCD_WR_REG(LCD_Reg);
	LCD_WR_DATA(LCD_RegValue);
}

void LCD_SetWindows(u16 xStar, u16 yStar,u16 xEnd,u16 yEnd)
{	
	LCD_WR_REG(SET_X_ADD_CMD);	
	LCD_WR_DATA(xStar>>8);
	LCD_WR_DATA(xStar);		
	LCD_WR_DATA(xEnd>>8);
	LCD_WR_DATA(xEnd);
	LCD_WR_REG(SET_Y_ADD_CMD);	
	LCD_WR_DATA(yStar>>8);
	LCD_WR_DATA(yStar);		
	LCD_WR_DATA(yEnd>>8);
	LCD_WR_DATA(yEnd);
	LCD_WR_REG(WR_RAM_CMD);	//ʼдGRAM				
}   

void LCD_Clear(u16 Color)
{
	u16 i,j;
	LCD_SetWindows(0,0,width-1,height-1);	
	LCD_CS=0;
	LCD_RS=1;
  for(i=0;i<width;i++)
	{
	  for (j=0;j<height;j++)
	   	{
			SPI_WriteByte(Color>>8);
			SPI_WriteByte(Color);
	    }
	}
	LCD_CS=1;
}

void LCD_Reset(void)
{	
	LCD_RESET=0;
	delay_ms(20);
	LCD_RESET=1;
	delay_ms(20);
}

void LCD_direction(u8 direction)
{ 
	switch(direction%4){		  
		case 0:						 	 		
			width=LCD_W;
			height=LCD_H;	
			LCD_WriteReg(0x36,0);//BGR==1,MY==0,MX==0,MV==0
		break;
		case 1:
			width=LCD_H;
			height=LCD_W;
			LCD_WriteReg(0x36,(1<<6)|(1<<5));//BGR==1,MY==1,MX==0,MV==1
		break;
		case 2:						 	 		
			width=LCD_W;
			height=LCD_H;		
			LCD_WriteReg(0x36,(1<<6)|(1<<7));//BGR==1,MY==0,MX==0,MV==0
		break;
		case 3:
			width=LCD_H;
			height=LCD_W;
			LCD_WriteReg(0x36,(1<<7)|(1<<5));//BGR==1,MY==1,MX==0,MV==1
		break;	
		default:break;
	}		
}

void LCD_Init(void)
{
	LCD_Reset(); //ʼ֮ǰλ
//*************ST7789Vʼ**********//
	delay_ms(120);
//------------------------Display Setting-------------------------------------//
	LCD_WR_REG(0x36);
	LCD_WR_DATA(0x00);
	LCD_WR_REG(0x3a);
	LCD_WR_DATA(0x55);
//-------------------ST7789V Frame rate setting-----------------//
	LCD_WR_REG(0xb2);
	LCD_WR_DATA(0x0c);
	LCD_WR_DATA(0x0c);
	LCD_WR_DATA(0x00);
	LCD_WR_DATA(0x33);
	LCD_WR_DATA(0x33);
	LCD_WR_REG(0xb7);
	LCD_WR_DATA(0x35);
//---------------------------------------------------//
	LCD_WR_REG(0xbb);
	LCD_WR_DATA(0x2B);
	LCD_WR_REG(0xc0);
	LCD_WR_DATA(0x2c);
	LCD_WR_REG(0xc2);
	LCD_WR_DATA(0x01);
	LCD_WR_REG(0xc3);
	LCD_WR_DATA(0x12);
	LCD_WR_REG(0xc4);
	LCD_WR_DATA(0x20);
	LCD_WR_REG(0xc6);
	LCD_WR_DATA(0x0f);
	LCD_WR_REG(0xd0);
	LCD_WR_DATA(0xa4);
	LCD_WR_DATA(0xa1);
/* Positive Voltage Gamma Control */
	LCD_WR_REG( 0xE0);
	LCD_WR_DATA(0xd0);
	LCD_WR_DATA(0x04);
	LCD_WR_DATA(0x0d);
	LCD_WR_DATA(0x11);
	LCD_WR_DATA(0x13);
	LCD_WR_DATA(0x2b);
	LCD_WR_DATA(0x3f);
	LCD_WR_DATA(0x54);
	LCD_WR_DATA(0x4c);
	LCD_WR_DATA(0x18);
	LCD_WR_DATA(0x0d);
	LCD_WR_DATA(0x0b);
	LCD_WR_DATA(0x1f);
	LCD_WR_DATA(0x23);
/* Negative Voltage Gamma Control */
	LCD_WR_REG( 0xE1);
	LCD_WR_DATA(0xd0);
	LCD_WR_DATA(0x04);
	LCD_WR_DATA(0x0c);
	LCD_WR_DATA(0x11);
	LCD_WR_DATA(0x13);
	LCD_WR_DATA(0x2c);
	LCD_WR_DATA(0x3f);
	LCD_WR_DATA(0x44);
	LCD_WR_DATA(0x51);
	LCD_WR_DATA(0x2f);
	LCD_WR_DATA(0x1f);
	LCD_WR_DATA(0x1f);
	LCD_WR_DATA(0x20);
	LCD_WR_DATA(0x23);
/*Sleep out*/
	LCD_WR_REG( 0x11);
	delay_ms(120);
/*Display on*/
	LCD_WR_REG( 0x29);		
//	LCD_BL=1;//	 
}

//
void main(void)
{ 
	u8 i=0;
	//STC_12LE5A60S2ƬP3.2ΪڿƱ
	//ͨC51Ƭ޴
	//P3M1 &= ~(1<<2),P3M0 |=  (1<<2); 
	SPI_init(); //ʼӲSPI
	//Һʼ
	LCD_Init();

	//ѭи	
	while(1)
	{
		LCD_direction(i++);
		LCD_Clear(WHITE);delay_ms(1000);
		LCD_Clear(BLACK);delay_ms(1000);
		LCD_Clear(RED);delay_ms(1000);
		LCD_Clear(GREEN);delay_ms(1000);
		LCD_Clear(BLUE);delay_ms(1000);
		if(i>3)
		{
			i=0;
		}
	}   
}
